"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const index_1 = require("./index");
const utils_1 = require("./utils");
const lodash_1 = require("lodash");
const relationPropsKey = "relationProps";
function getRelationString(label, relationType, relationProps) {
    let relationPropsStr = "";
    if (relationProps) {
        relationPropsStr = lodash_1.keys(relationProps)
            .map(key => `${key}: {${relationPropsKey}}.${key}`)
            .join(", ");
    }
    if (relationType.any) {
        return `-[r:${label} {${relationPropsStr}}]-`;
    }
    return `${!relationType.out && !relationType.any ? "<" : ""}-[r:${label} {${relationPropsStr}}]-${!relationType.out && !relationType.any ? "" : ">"}`;
}
function get(instance, label, relationType, props, relationProps) {
    return __awaiter(this, void 0, void 0, function* () {
        const { where, flatProps } = utils_1.prepareWhere({ b: props, r: relationProps }, [
            "b",
            "r",
        ]);
        const relationString = getRelationString(label, relationType);
        const result = yield index_1.default.run(`
    MATCH (a:${this.src.label} {guid:{_srcGuid}})${relationString}(b:${this.dest.label})
    ${where}
    RETURN b, r
  `, Object.assign({ _srcGuid: instance.props.guid }, flatProps));
        return Promise.resolve(result.map(p => {
            const instance = this.dest._createModelInstance(p.b);
            instance.relationProps = p.r;
            return instance;
        }));
    });
}
exports.get = get;
function remove(instance, label, relationType, props, relationProps) {
    return __awaiter(this, void 0, void 0, function* () {
        const { where, flatProps } = utils_1.prepareWhere({ b: props, r: relationProps }, [
            "b",
            "r",
        ]);
        const relationString = getRelationString(label, relationType);
        const result = yield index_1.default.run(`
    MATCH (a:${this.src.label} {guid:{_srcGuid}})${relationString}(b:${this.dest.label})
    ${where}
    DELETE r
    `, Object.assign({ _srcGuid: instance.props.guid }, flatProps));
        return Promise.resolve(result._stats);
    });
}
exports.remove = remove;
function create(instance, label, relationType, propsArray, relationProps) {
    return __awaiter(this, void 0, void 0, function* () {
        const destInstances = [];
        const relationString = getRelationString(label, relationType, relationProps);
        for (const props of propsArray) {
            const destInstance = yield this.dest.create(props);
            destInstances.push(destInstance);
            yield index_1.default.run(`
      MATCH
        (a:${this.src.label} {guid:{srcGuid}}),
        (b:${this.dest.label} {guid:{destGuid}})
      MERGE (a)${relationString}(b)
    `, {
                srcGuid: instance.props.guid,
                destGuid: destInstance.props.guid,
                relationProps,
            });
        }
        return Promise.resolve(destInstances);
    });
}
exports.create = create;
function add(instance, label, relationType, instances, relationProps) {
    return __awaiter(this, void 0, void 0, function* () {
        const relationString = getRelationString(label, relationType, relationProps);
        let relationshipsCreated = 0;
        for (const destInstance of instances) {
            const result = yield index_1.default.run(`
      MATCH
        (a:${this.src.label} {guid:{srcGuid}}),
        (b:${this.dest.label} {guid:{destGuid}})
      MERGE (a)${relationString}(b)
    `, {
                srcGuid: instance.props.guid,
                destGuid: destInstance.props.guid,
                relationProps,
            });
            relationshipsCreated += result._stats.relationshipsCreated;
        }
        return Promise.resolve(relationshipsCreated);
    });
}
exports.add = add;
function count(instance, label, relationType, props, relationProps) {
    return __awaiter(this, void 0, void 0, function* () {
        const { where, flatProps } = utils_1.prepareWhere({ b: props, r: relationProps }, [
            "b",
            "r",
        ]);
        const relationString = getRelationString(label, relationType);
        const result = yield index_1.default.run(`
    MATCH (a:${this.src.label} {guid:{_srcGuid}})${relationString}(b:${this.dest.label})
    ${where}
    RETURN COUNT(b) as b
  `, Object.assign({ _srcGuid: instance.props.guid }, flatProps));
        const low = result._raw.records[0]._fields[0].low || -1;
        return Promise.resolve(low);
    });
}
exports.count = count;
function update(instance, label, relationType, props, whereProps, relationProps, whereRelationProps) {
    return __awaiter(this, void 0, void 0, function* () {
        const { where, flatProps } = utils_1.prepareWhere({ b: whereProps, r: whereRelationProps }, ["b", "r"]);
        const { str: setPropsStr, newProps } = utils_1.prepareSet({
            b: props,
            r: relationProps,
        }, ["b", "r"]);
        const relationString = getRelationString(label, relationType);
        const result = yield index_1.default.run(`
    MATCH (a:${this.src.label} {guid:{_srcGuid}})${relationString}(b:${this.dest.label})
    ${where}
    SET ${setPropsStr}
    RETURN b
  `, Object.assign({ _srcGuid: instance.props.guid }, flatProps, newProps));
        return Promise.resolve(result.map(a => this.dest._createModelInstance(a.b)));
    });
}
exports.update = update;
