"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const uuid = require("uuid");
const index_1 = require("./index");
const utils_1 = require("./utils");
class Model {
    constructor(label) {
        this.label = label;
        this.relations = [];
    }
    beforeCreate(props) {
        return props;
    }
    afterCreate(instance) {
        return instance;
    }
    beforeFind(props) {
        return props;
    }
    afterFind(instance) {
        return instance;
    }
    beforeUpdate(props, newProps) {
        return { props, newProps };
    }
    afterUpdate(instance) {
        return instance;
    }
    _createModelInstance(props) {
        let instance = this.modelInstanceClass
            ? new this.modelInstanceClass(props)
            : new index_1.ModelInstance(props);
        this.relations.forEach(r => (instance = r.addFunctionsToInstance(instance)));
        return instance;
    }
    create(props) {
        return __awaiter(this, void 0, void 0, function* () {
            let defaultProps = lodash_1.mapValues(this.modelInstanceClass
                ? this.modelInstanceClass.prototype._defaultProps
                : {}, prop => (typeof prop === "function" ? prop() : prop));
            let p = this.beforeCreate(Object.assign({ guid: uuid.v4() }, defaultProps, props));
            p = Object.assign({}, p);
            const result = yield index_1.default.run(`
        CREATE (n:${this.label} {p})
        RETURN n
      `, { p });
            if (!result || result.length !== 1) {
                throw new Error(`Create didn't work, cmd: "CREATE (n:${this.label} {p}) RETURN n" with params: ${JSON.stringify({ p })}`);
            }
            return this.afterCreate(this._createModelInstance(result[0].n));
        });
    }
    findByGuid(guid) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield index_1.default.run(`
        MATCH (n:${this.label} {guid:{guid}})
        RETURN n
      `, { guid });
            if (!result || result.length > 1) {
                throw new Error(`Match didn't work, cmd: "MATCH (n:${this.label} {guid:{guid}}) RETURN n" with params: ${JSON.stringify({
                    guid,
                })}`);
            }
            if (result.length === 0) {
                return null;
            }
            return this._createModelInstance(result[0].n);
        });
    }
    delete(props, detach = false) {
        return __awaiter(this, void 0, void 0, function* () {
            const { where, flatProps } = utils_1.prepareWhere(props, "n");
            const result = yield index_1.default.run(`
        MATCH (n:${this.label})
        ${where}
        ${detach ? " DETACH " : ""} DELETE n
      `, flatProps);
            return result._stats.nodesDeleted;
        });
    }
    find(props) {
        return __awaiter(this, void 0, void 0, function* () {
            const p = this.beforeFind(props);
            const { where, flatProps } = utils_1.prepareWhere(p, "n");
            let result = yield index_1.default.run(`
        MATCH (n:${this.label})
        ${where}
        RETURN n
      `, flatProps);
            return result.map(p => this.afterFind(this._createModelInstance(p.n)));
        });
    }
    findOne(props) {
        return __awaiter(this, void 0, void 0, function* () {
            const p = this.beforeFind(props);
            const { where, flatProps } = utils_1.prepareWhere(p, "n");
            let result = yield index_1.default.run(`
        MATCH (n:${this.label})
        ${where}
        RETURN n
      `, flatProps);
            if (result.length) {
                return this.afterFind(this._createModelInstance(result[0].n));
            }
            return Promise.resolve(null);
        });
    }
    update(props, newProps) {
        return __awaiter(this, void 0, void 0, function* () {
            const params = this.beforeUpdate(props, newProps);
            const { where, flatProps } = utils_1.prepareWhere(params.props, "n");
            const { str: setPropsStr, newProps: _newProps } = utils_1.prepareSet(params.newProps, "n");
            let result = yield index_1.default.run(`
        MATCH (n:${this.label})
        ${where}
        SET ${setPropsStr}
        RETURN n
      `, Object.assign({}, flatProps, _newProps));
            return result.map(p => this.afterUpdate(this._createModelInstance(p.n)));
        });
    }
}
exports.Model = Model;
